import React from 'react';
import { Table } from 'antd';
import { TableWrapper } from './TableWrapper';
import { TableHeader } from './TableHeader';
import { TableHeaderRow } from './TableHeaderRow';
import { CommonSearch } from '../CommonSearch/CommonSearch';
import { reOrder } from 'utils/reOrder';
import { Form , message } from 'antd';

import { filterFalsify } from '../../utils/filterFalsify';
import $http from 'tms-request';

export const CommonTable = Form.create()(
    class CommonTable extends React.PureComponent {
        constructor(props) {
            super();
            const {columns , pageSize = 20} = props;
            this.state = {
                columns ,
                filterColumns: columns.map(column => column.title) ,
                filterColumnsSearchText: '' ,
                current: 1 ,
                pageSize ,
                total: 0 ,
                list: [] ,
                loading: false
            };
        }

        /**
         *
         * @return {void}
         */
        async componentDidMount() {
            await this.handlePageChange();
        }

        toggleLoading = () => {
            this.setState(prevState => ({
                loading: !prevState.loading
            }));
        };

        handleFilterColumnsReset = () => {
            const {columns} = this.state;
            this.setState({
                filterColumns: columns.map(column => column.title)
            });
        };

        handleFilterColumnsCheck = filterColumns => {
            this.setState({filterColumns});
        };

        handleFilterColumnsSearch = e => {
            this.setState({filterColumnsSearchText: e.target.value});
        };

        static getDerivedStateFromProps(nextProps , prevState) {
            const {columns , updateColumns = false} = nextProps;
            const result = {};
            if (
                Array.isArray(columns) &&
                prevState.columns !== columns &&
                updateColumns
            ) {
                result.columns = columns;
            }

            if (
                nextProps.pagination &&
                prevState.pagination !== nextProps.pagination
            ) {
                result.pagination = nextProps.pagination;
            }
            return Object.keys(result) ? result : null;
        }

        components = {
            header: {
                cell: TableHeader ,
                row: TableHeaderRow
            }
        };

        //添加自定义表头的props
        handleProcessColumns = columns => {
            const result = columns.map((column , columnIndex) => {
                const newColumn = {...column};
                newColumn.onHeaderCell = column => ({
                    minWidth: column.minWidth ,
                    width: column.width ,
                    maxWidth: column.maxWidth ,
                    draggable: column.draggable ,
                    columnIndex
                });
                return newColumn;
            });
            return result.filter(v => {
                return this.state.filterColumns.includes(v.title);
            });
        };

        handleDragEnd = result => {
            if (!result.destination) {
                return;
            }
            const columns = reOrder(
                this.state.columns ,
                result.source.index ,
                result.destination.index
            );

            this.setState({
                columns
            });
        };

        onHeaderRow = () => ({
            onDragEnd: this.handleDragEnd
        });

        getRowKey = (record , index) => record.id || index;

        handlePageChange = async (
            current = this.state.current ,
            pageSize = this.state.pageSize
        ) => {
            await this.fetchData({
                current ,
                pageSize
            });
        };

        handleReset = () => {
            this.props.form.resetFields();
        };

        fetchData = async pagination => {
            const {
                fetchUrl ,
                method = 'get' ,
                form ,
                requestData = {} ,
                requestHeader = {}
            } = this.props;

            if (!fetchUrl) {
                return;
            }

            const formData = filterFalsify(form.getFieldsValue());
            this.toggleLoading();
            try {
                const data = {
                    ...pagination ,
                    ...formData ,
                    ...requestData
                };

                const result = await $http.request({
                    method ,
                    url: fetchUrl ,
                    data ,
                    headers: {
                        ...requestHeader
                    }
                });
                this.setState(result);
            } catch (e) {
                message.error(e.message);
            } finally {
                this.toggleLoading();
            }
        };

        render() {
            const {
                pagination = true ,
                paginationOptions = {} ,
                searchItems ,
                fullTextSearch = true ,
                outerFilter = true ,
                onFullTextSearch = () => {} ,
                onFullTextSearchChange = () => {} ,
                form ,
                ...restProps
            } = this.props;

            const {current , pageSize , total , list , loading} = this.state;
            const columns = this.handleProcessColumns(this.state.columns);

            const {filterColumns , filterColumnsSearchText} = this.state;

            let innerPagination = false;

            if (pagination) {
                innerPagination = {
                    current ,
                    pageSize ,
                    total ,
                    onChange: this.handlePageChange ,
                    ...paginationOptions
                };
            }

            return (
                <TableWrapper>
                    <CommonSearch
                        loading={loading}
                        searchItems={searchItems}
                        outerFilter={outerFilter}
                        filterColumns={filterColumns}
                        filterColumnsSearchText={filterColumnsSearchText}
                        columns={this.state.columns}
                        pagination={innerPagination}
                        fullTextSearch={fullTextSearch}
                        onFullTextSearch={onFullTextSearch}
                        onFullTextSearchChange={onFullTextSearchChange}
                        onFormReset={this.handleReset}
                        onFormSearch={this.handlePageChange}
                        form={form}
                        onFilterColumnCheck={this.handleFilterColumnsCheck}
                        onFilterColumnsReset={this.handleFilterColumnsReset}
                        onFilterColumnSearch={this.handleFilterColumnsSearch}
                    />
                    <Table
                        rowKey={this.getRowKey}
                        {...restProps}
                        columns={columns}
                        dataSource={list}
                        components={this.components}
                        onHeaderRow={this.onHeaderRow}
                        pagination={innerPagination}
                    />
                </TableWrapper>
            );
        }
    }
);
